#!/usr/bin/env python3
import os
import sys
import signal

import random

def handle_sigusr1(signum, frame):
    global processed_lines
    print(f"Processed {processed_lines} lines", file=sys.stderr)

def main():
    global processed_lines
    processed_lines = 0

    pipe_a_read, pipe_a_write = os.pipe()
    pipe_b_read, pipe_b_write = os.pipe()
    pipe_c_read, pipe_c_write = os.pipe()

    if os.fork() == 0:
        os.dup2(pipe_a_write, sys.stdout.fileno())
        os.close(pipe_a_read)
        os.close(pipe_a_write)
        os.close(pipe_b_read)
        os.close(pipe_b_write)
        os.close(pipe_c_read)
        os.close(pipe_c_write)

        random_n = random.randint(120, 180)
        os.execvp("python3", ["python3", "generator", str(random_n)])

    if os.fork() == 0:
        os.dup2(pipe_b_read, sys.stdin.fileno())
        os.dup2(pipe_c_write, sys.stdout.fileno())
        os.close(pipe_a_read)
        os.close(pipe_a_write)
        os.close(pipe_b_read)
        os.close(pipe_b_write)
        os.close(pipe_c_read)
        os.close(pipe_c_write)

        os.execvp("/usr/bin/bc", ["bc"])

    os.close(pipe_a_write)
    os.close(pipe_b_read)
    os.close(pipe_c_write)

    signal.signal(signal.SIGUSR1, handle_sigusr1)

    try:
        while True:
            line = os.read(pipe_a_read, 1024).decode()
            if not line:
                break

            os.write(pipe_b_write, line.encode())

            result = os.read(pipe_c_read, 1024).decode().strip()
            print(f"{line.strip()} = {result}")
            processed_lines += 1

    except KeyboardInterrupt:
        pass
    finally:
        os.close(pipe_a_read)
        os.close(pipe_b_write)
        os.close(pipe_c_read)

if __name__ == "__main__":
    main()